-- -------------------------------------------------------------------- --
--                                                                      --
--            (C) Kerio Technologies Inc. All rights reserved.          --
--                                                                      --
-- -------------------------------------------------------------------- -- 

--  Load generated variables and set global variables.

set scriptFolderMac to ""
set scriptFolderQuotedPOSIX to ""
tell application "Finder"
	set pathToThisFile to path to me
	set scriptFolderMac to (container of pathToThisFile) as alias as string
	set scriptFolder to POSIX path of scriptFolderMac
	set scriptFolderQuotedPOSIX to quoted form of scriptFolder
end tell

do shell script "osacompile -o " & scriptFolderQuotedPOSIX & "preflight_incl.scpt " & scriptFolderQuotedPOSIX & "preflight_incl"
global homeDir
global PreflightIncl, pref2004, pref2008
set PreflightIncl to load script file (scriptFolderMac & "preflight_incl.scpt")
set pref2004 to "/Library/Preferences/Microsoft/Entourage Preferences"
set pref2008 to "/Library/Preferences/Microsoft/Office 2008/Entourage Preferences"

-- Functions.

to fileExists(fileName)
	try
		set testFile to fileName as alias
		return true
	on error
		return false
	end try
end fileExists

to getHomeDir()
	return homeDir
end getHomeDir

to setHomeDir()
	set scriptFolder to ""
	tell application "Finder"
		set pathToThisFile to path to me
		set scriptFolder to (container of pathToThisFile) as alias as string
	end tell
	
	try
		set homeDir to first paragraph of (read alias (scriptFolder & "homePath")) as text
	on error errStr
		set errMsg to "Cannot get home path. Error: " & errStr & " Exiting."
		log (errMsg)
		tell application "System Events"
			activate
			display dialog errMsg buttons {"OK"} default button 1
		end tell
		error
	end try
end setHomeDir

to isIdentityDialog()
	set homeDir to getHomeDir()
	
	set prefEntourage to POSIX file (homeDir & pref2004)
	if not fileExists(prefEntourage) then
		set prefEntourage to POSIX file (homeDir & pref2008)
		if not fileExists(prefEntourage) then
			return true
		end if
	end if
	
	try
		set profileSelection to read alias prefEntourage from 29 to 30
	on error
		return true
	end try
	
	if (ASCII number (character 1 of profileSelection)) is equal to 0 then
		return false
	else
		return true
	end if
end isIdentityDialog

on getRunning(appl)
	set res to false
	try
		tell application "System Events" to set res to (name of processes) contains appl
	end try
end getRunning

to createNewKMSAccount()
	set EmailAddress to (PreflightIncl's shortName) & "@" & (PreflightIncl's DomainName)
	set PublicFolderServer to (PreflightIncl's KerioMailServer) & "/public/"
	set KerioMailServerLDAP to PreflightIncl's KerioMailServer
	set LDAPRequiresAuth to true
	set LDAPServerPort to PreflightIncl's LDAPServerPort
	
	if PreflightIncl's LDAPServerPort = 0 then
		set PreflightIncl's LDAPServerSSL to false
		set KerioMailServerLDAP  to ""
		set LDAPRequiresAuth to false
		set LDAPServerPort to 3268
	end if
	
	tell application "Microsoft Entourage"
		set newExchangeAccount to make new Exchange account with properties {name:EmailAddress, Exchange ID:PreflightIncl's shortName, domain:PreflightIncl's DomainName, full name:PreflightIncl's fullName, email address:EmailAddress, search base:"", Exchange server settings:{address:PreflightIncl's KerioMailServer, requires SSL:PreflightIncl's KerioMailServerSSL, port:PreflightIncl's KerioMailServerPort}, public folder server settings:{address:PublicFolderServer, requires SSL:PreflightIncl's KerioMailServerSSL, port:PreflightIncl's KerioMailServerPort}, LDAP server settings:{address:KerioMailServerLDAP, LDAP requires authentication:LDAPRequiresAuth, requires SSL:PreflightIncl's LDAPServerSSL, port:LDAPServerPort, maximum entries:100}}
		set LDAP requires authentication of newExchangeAccount to LDAPRequiresAuth
		return newExchangeAccount 
	end tell
	
end createNewKMSAccount

to isEntourageInstalled()
	try
		tell application "Finder"
			set appId to application file id "com.microsoft.Entourage"
			log ("Entourage installed:" & appId)
		end tell
		return true
	on error
		log ("Cannot determine Entourage bundle ID. Assuming Entourage is not installed.")
		return false
	end try
end isEntourageInstalled

to log (logString)
	set logFile to getHomeDir() & "/Library/Logs/kerio-entourage-config-tool.log"
	
	try
		set logEntry to ((do shell script "date +'%Y-%m-%d %H:%M:%S'" with administrator privileges) as string) & " " & logString
		do shell script ("echo " & logEntry & " >> " & logFile) with administrator privileges
	end try
end log

to isEntourageResponding()
	try
		tell application "Microsoft Entourage"
			set allEntourageWindows to get every window
			if (count of allEntourageWindows's items) > 2 then
				return false
			end if
		end tell
	on error
		return false
	end try
	return true
end isEntourageResponding

--
-- Main start
--

setHomeDir()

log ("AppleScript started.")

--  Check whether services are running on Kerio MailServer.
if PreflightIncl's KerioMailServerPort = 0 then
	set errMsg to "Kerio MailServer has no available HTTP ports. No Entourage WebDAV accounts will be created. Contact your administrator about the issue."
	log (errMsg)
	tell application "System Events"
		activate
		display dialog errMsg buttons {"OK"} default button 1
	end tell
	error
end if

--  Check installed Entourage.
if not isEntourageInstalled() then
	set errMsg to "Microsoft Entourage 2004/2008 does not seem to be properly installed. Exiting."
	log (errMsg)
	tell application "System Events"
		activate
		display dialog errMsg buttons {"OK"} default button 1
	end tell
	error
end if

--  Check for "Choose identity on start" dialog.
repeat
	set isEntourageRunning to getRunning("Microsoft Entourage")
	set isIdentDialog to isIdentityDialog()
	
	log ("Is Entourage running: " & isEntourageRunning & ".")
	log ("Is identity dialog displayed: " & isIdentDialog & ".")
	
	if ((not isEntourageRunning) and isIdentDialog) then
		try
			tell application "System Events"
				activate
				with timeout of 10000 seconds
					display dialog "To configure Kerio MailServer account, please start Microsoft Entourage with specific identity and press Continue." buttons {"Cancel", "Continue"} default button 2
				end timeout
			end tell
		on error
			log ("Setup was canceled.")
			error -- Cancel pressed
		end try
	else
		exit repeat
	end if
end repeat

--  Install certificate.
if PreflightIncl's KerioMailServerSSL is true then
	log ("Trying to install SSL certificate.")
	try
		set shellResult to do shell script "certtool i " & scriptFolderQuotedPOSIX & PreflightIncl's KerioMailServer & ".cer v k=/System/Library/Keychains/X509Anchors" with administrator privileges
		set shellResult to do shell script "certtool i " & scriptFolderQuotedPOSIX & PreflightIncl's KerioMailServer & ".cer v k=" & getHomeDir() & "/Library/Keychains/login.keychain" with administrator privileges
		log ("SSL certificate installed.")
	on error errStr
		log ("Warning: SSL certificate not installed. " & return & errStr)
	end try
end if

-- Test whether there is no open dialog window in Entourage which will block keyboard input.
log ("Trying whether Entourage is responsive to AppleScript calls.")
repeat
	set isEntResponding to isEntourageResponding()
	log ("Is Entourage responding: " & isEntResponding  & ".")
	
	if not isEntResponding then
		try
			tell application "System Events"
				activate
				with timeout of 10000 seconds
					display dialog "To configure Kerio MailServer account, please close all dialog windows in Entourage, leave only main Entourage window opened, and press Continue." buttons {"Cancel", "Continue"} default button 2
				end timeout
			end tell
		on error
			log ("Setup was canceled.")
			error -- Cancel pressed
		end try
	else
		exit repeat
	end if
end repeat


--  Configure Kerio MailServer account in Entourage.
log ("Going to Entourage setup.")
try
	tell application "Microsoft Entourage"
		set isAlreadyConfiguredAccount to false
		set exchangeAccounts to every Exchange account
		repeat with theAccount in exchangeAccounts
			if ({address:PreflightIncl's KerioMailServer} is in Exchange server settings of theAccount) and ((get domain of theAccount) is equal to PreflightIncl's DomainName) and ((get Exchange ID of theAccount) is equal to PreflightIncl's shortName) then
				set isAlreadyConfiguredAccount to true
				exit repeat
			end if
		end repeat
		
		if isAlreadyConfiguredAccount then
			delete theAccount
		end if
	end tell
	
	if isAlreadyConfiguredAccount then
		log ("Removing already existing account.")
	end if
	
	log ("Creating new Entourage account.")
	
	tell application "Microsoft Entourage"
		set newExchangeAccount to my createNewKMSAccount()
	end tell
	log ("Entourage configuration finished.")

	if PreflightIncl's LDAPServerPort = 0 then
		set errMsg to "Entourage WebDAV account has been created. However, there are no LDAP ports available on Kerio MailServer. Entourage Directory will not work."
		log (errMsg)
		tell application "System Events"
			activate
			display dialog errMsg buttons {"OK"} default button 1
		end tell
	end if
	
on error errStr
	log ("Entourage failed when executing AppleScript. " & errStr)
	tell application "System Events"
		activate
		display dialog "Error occured in the tool when accessing Entourage. Restart Entourage and then run the tool again." buttons {"OK"} default button 1
	end tell
	error
end try

log ("AppleScript finished." & return)

return 0
