###########################################################################
#                                                                         #
#             (C) Kerio Technologies Inc. All rights reserved.            #
#                                                                         #
###########################################################################

################################# Imports #################################

# Standard imports.
import sys
import os
import uuid
import plistlib
import pwd
import traceback
import platform

from ScriptingBridge import *

# Import server-generated variables.
from configure_include import *

# Globals.
LDAPServerConfigs = "LDAP Server Configs"
LDAPSearchNode = "Search Node Custom Path Array"
CalendarDirectory = "Library/Calendars"
gLogFile = "Library/Logs/kerio-ical-config-tool.log"

############################# Open Directory ##############################

# Sanity check.
def ensureLDAPServerConfigsKeyExists(DSLDAPv3Plist):
	if not DSLDAPv3Plist.has_key(LDAPServerConfigs):
		DSLDAPv3Plist[LDAPServerConfigs] = []

# Sanity check.
def ensureLDAPSearchNodePolicyKeyExists(ContactsNodeConfig):
	if not ContactsNodeConfig.has_key(LDAPSearchNode):
		ContactsNodeConfig[LDAPSearchNode] = []

# Get user@name.
def getKMSUserNameAtDomain():
	return KMSUser + "@" + KMSDomain
	
# Get Mac OS X version.
def isLeopard():
	osVersionString = platform.mac_ver()[0]
	releaseVersion = int(osVersionString.split('.')[1])
	if releaseVersion == 5:
		return True
	else:
		return False
	
# Get home directory (usual functions return /var/root/...).
# We want something like "/User/username/".
def getHomeDir():

	pathFile = open(sys.path[0] + "/homePath", "r")
	line = pathFile.readline()
	line = line.rstrip("\n") + "/"
	return line
	
	conUser = getConsoleUser()
	homeDir = os.popen("sudo dscl /Local/Default -read /Users/"
		+ conUser + " NFSHomeDirectory").readlines()

	if len( homeDir ) > 0:
		start = homeDir[0].find("/")
		stop = len(homeDir[0]) - 1
		return homeDir[0][start:stop] + "/"
	else:
		print >> sys.stderr, "Cannot determine user home directory."
		return ""

# Initialize user and domain name from login name.
def initializeUserAndDomain(loginName):
	global KMSUser
	global KMSDomain

	userDomainArray = loginName.split("@");

	if len( userDomainArray ) > 0:
		KMSUser = userDomainArray[0]

	if len( userDomainArray ) > 1:
		KMSDomain = userDomainArray[1]
	else:
		KMSDomain = KMSPrimaryDomain

# Return hostname and port from DSLDAPv3Plist entry.
def getServerEntryParameters(ldapServerPlistEntry):
	host = ldapServerPlistEntry.get("Server")
	port = ldapServerPlistEntry.get("Port Number")
	return (host, port)

# Try to rule out whether username/password are correct.
def tryConnect(password):

	# Construct URL	
	httpUrl = "http"
	if KMSHTTPisSSL == True:
		httpUrl = httpUrl + "s"
	httpUrl = httpUrl + "://" + KMSHostName + ":" + repr(KMSHTTPPort)

	print >> sys.stderr, "\nTrying to connect to " +\
		httpUrl + " as " + getKMSUserNameAtDomain() + "."

	try:
		userNameAndPassword = getKMSUserNameAtDomain() + ":" + password
		userNameAndPassword = userNameAndPassword.replace('"', r'\"')
		userNameAndPassword = userNameAndPassword.replace('$', r'\$')

		resultFromKMS = os.popen("curl " + httpUrl + "/setup/entourage -u \"" + \
			userNameAndPassword	+ "\" -k --connect-timeout 30" ).readlines()

		l = len( resultFromKMS )
		if l == 0:
			print >> sys.stderr, "Server unreachable."
			return -2		
		elif l < 5:
			print >> sys.stderr, "Username/password incorrect."
			return -1
		else:
			print >> sys.stderr, "Username/password seems to be correct."
			return 0
		
	except:
		print >> sys.stderr, "The server could not fulfill the request."
		return -2		

# Perform shell command with no error and output messages.
def performOSCommand(command):
	os.system(command + " > /dev/null 2>&1")

# Return index (0..n) of existing hostname entry in DSLDAPv3Plist.
# Return -1 if no such entry found.
def findKmsEntryInTheList(DSLDAPv3Plist):
	index = 0
	for theLDAPServer in DSLDAPv3Plist.get(LDAPServerConfigs):
		host, port = getServerEntryParameters(theLDAPServer)
		if host.lower() == KMSHostName.lower():
			return index
		index = index + 1
	return -1

# Add entries to DSLDAPv3PlugInMappings.plist.
def addKMSToDirectoryConfiguration(DSLDAPv3Plist, password):
	# insert attribute and record maps from external file
	
	try:
		dsLdapTemplateFile = open(
			sys.path[0] + "/DSLDAPv3PlugInMappings.plist", 
			"r")
		dsLdapTemplateString = dsLdapTemplateFile.read()
		dsLdapFiledString = dsLdapTemplateString.replace("%s", KMSDomain)
		theEntry = plistlib.readPlistFromString( dsLdapFiledString )

	except:
		print >> sys.stderr, "Unable to open DSLDAPv3PlugInMappings.plist from " +\
			"installation directory."
		return False

	# insert other attributes
	theEntry["Server"] = KMSHostName
	theEntry["Port Number"] = KMSLDAPPort
	theEntry["SSL"] = KMSLDAPisSSL
	theEntry["Template Search Base Suffix"] = "dc=" + KMSDomain

	theEntry["Secure Use"] = True
	theEntry["Server Account"] = getKMSUserNameAtDomain()
	theEntry["Server Password"] = plistlib.Data(password)

	theEntry["Configuration UUID"] = getGuid()
	theEntry["Enable Use"] = True
	theEntry["Map Search Base"] = "cn=config, dc=" + KMSDomain
	theEntry["Server Mappings"] = False
	theEntry["Template Name"] = "Open Directory Server"
	theEntry["Template Version"] = "10.4"
	theEntry["UI Name"] = "Kerio MailServer"
	theEntry["Use DNS replicas"] = False
	theEntry["Delay Rebind Try in seconds"] = 120
	theEntry["Idle Timeout in minutes"] = 2
	theEntry["OpenClose Timeout in seconds"] = 15
	theEntry["Search Timeout in seconds"] = 120

	DSLDAPv3Plist[LDAPServerConfigs].append( theEntry )

	return True

# Return index (0..n) of serverHostName entry in 
# ContactsNodeConfig.plist. Return -1 if no such entry found.
def searchNodeEntryInTheList(ContactsNodeConfig, serverHostName):
	index = 0
	for theEntry in ContactsNodeConfig.get("Search Node Custom Path Array"):
		searchStringNode = "/LDAPv3/" + serverHostName
		if searchStringNode.lower() == theEntry.lower():
			return index
		index = index + 1
	return -1

# Configure ContactsNodeConfig.plist.
def configureSearchContactsNode(serverHostName):
	ContactsNodeConfigPath = "/Library/Preferences/DirectoryService/ContactsNodeConfig.plist"
	ContactsNodeConfig = plistlib.Plist.fromFile( ContactsNodeConfigPath )

	ensureLDAPSearchNodePolicyKeyExists(ContactsNodeConfig)
	searchNodeEntryInTheListIdx = searchNodeEntryInTheList(ContactsNodeConfig, serverHostName)
	ContactsNodeConfig["Search Policy"] = 3

	if searchNodeEntryInTheListIdx == -1:
		print >> sys.stderr, "Creating new KMS LDAP server entry in Searching Contacts list."
		ContactsNodeConfig[LDAPSearchNode].append( "/LDAPv3/" + serverHostName)
	else:
		print >> sys.stderr, "KMS LDAP server already in Searching Contacts list."

	print >> sys.stderr, "Writing Searching Contacts list configuration."
	ContactsNodeConfig.write(ContactsNodeConfigPath)

# Configure SSL certificate with OpenSSL.
def configureSecureLDAP():
	# copy certificate file from Installer package to system location
	print >> sys.stderr, "Copying certificate to OpenSSL certificate store."
	CertsDir = "/System/Library/OpenSSL/certs"
	if not os.path.exists( CertsDir ):
		os.mkdir( CertsDir )

	KMScert = KMSHostName + ".cer"
	KMScertFile = CertsDir + "/" + KMScert
	if os.path.exists(KMScertFile):
		os.remove(KMScertFile)
	
	# dist/KMSInstaller.app/Contents/Resources
	sourcePath = sys.path[0] + "/../../../../" + KMScert
	sourcePath = sourcePath.replace(" ", "\ ")
	
	performOSCommand("sudo cp " + sourcePath + " " + CertsDir)

	# modify ldap.conf
	LdapConf = "/etc/openldap/ldap.conf"
	performOSCommand("sudo cp " + LdapConf + " " + LdapConf + ".bak")
	performOSCommand("sudo sh -c 'echo TLS_REQCERT never > " + LdapConf + "'")
	performOSCommand("sudo sh -c 'grep -v \"^TLS_REQCERT\" " + LdapConf + ".bak >> "\
+ LdapConf + "'")
	
	return
	
	# is there already our record?
	KMSCertLdapConfRecord = "TLS_CACERT " + CertsDir + "/" + KMScert
	LdapConfFile = open(LdapConf, "r")
	while True:
		line = LdapConfFile.readline()
		if not line: break
		if line.startswith(KMSCertLdapConfRecord):
			print >> sys.stderr, "OpenSSL configuration already knows KMS\
 certificate."
			return

	print >> sys.stderr, "KMS certificate was not found in the system, adding\
 KMS certificate to OpenSSL configuration " + LdapConf + "."
	LdapConfFile = open(LdapConf, "a")	# append to ldap.conf
	LdapConfFile.write("\n" + KMSCertLdapConfRecord + "\n")
	LdapConfFile.close()

	return
	
# Main Directory Services configuration.
def configureDirectoryServices(loginName, password):
	if KMSLDAPisSSL == True:
		print >> sys.stderr, "Configuring OpenSSL."
		configureSecureLDAP()
	
	DSLDAPv3PlistPath = "/Library/Preferences/DirectoryService\
/DSLDAPv3PlugInConfig.plist"
	DSLDAPv3Plist = plistlib.Plist.fromFile(DSLDAPv3PlistPath)
	version = DSLDAPv3Plist.get("LDAP PlugIn Version")
	print >> sys.stderr, version

	# Is KMS entry already in DSLDAPv3...plist?
	ensureLDAPServerConfigsKeyExists(DSLDAPv3Plist)
	kmsEntryInTheList = findKmsEntryInTheList(DSLDAPv3Plist)
	if not (kmsEntryInTheList == -1):
		# Hostname and port configured in Directory Services are the same
		# as KMS parameters.
		port = DSLDAPv3Plist[LDAPServerConfigs][kmsEntryInTheList]["Port Number"]
		uiName = DSLDAPv3Plist[LDAPServerConfigs][kmsEntryInTheList]["UI Name"]
		if (KMSLDAPPort != port) and (uiName != "Kerio MailServer"):
			print >> sys.stderr, "Another Open Directory server on " + \
				KMSHostName + ":" + repr(port) + " is configured. " + \
				"KMS LDAP cannot be configured."
			return -3

		print >> sys.stderr, "LDAP server with " + KMSHostName + \
			" host name and " + repr(KMSLDAPPort) + " port already" + \
			" configured in Directory Services. Removing that entry."
		DSLDAPv3Plist[LDAPServerConfigs].pop(kmsEntryInTheList)

	print >> sys.stderr, "Adding " + KMSHostName + " host name with " + \
		repr(KMSLDAPPort) + " port to Directory Services."
	addKMSToDirectoryConfiguration(DSLDAPv3Plist, password)

	print >> sys.stderr, "Writing Directory Services configuration."
	DSLDAPv3Plist.write(DSLDAPv3PlistPath)

	configureSearchContactsNode(KMSHostName)

	print >> sys.stderr, "Restarting Directory Services."
	if isLeopard() == True:
		performOSCommand("sudo killall DirectoryService")
	else:
		performOSCommand("sudo killall -QUIT DirectoryService")
		
	return 0 

################################## iCal ###################################

# Get http(s)://hostname:port/caldav/users/domain/user/
def getCalDAVPrincipalURL():
	protocol = "http"
	if (KMSHTTPisSSL == True):
		protocol = protocol + "s"

	principalURL = protocol + "://" + KMSHostName + ":" + repr(KMSHTTPPort) \
		+ "/caldav/users/" + KMSDomain + "/" + KMSUser + "/"
	principalURL = principalURL.lower()

	return 	principalURL

# Get console user name
def getConsoleUser():
	# get user name logged to the Mac OS X session console
	macOSUserName = os.popen("ls -l /dev/console |\
		awk '{ print $3 }'").readlines()
	if len( macOSUserName ) < 1:
		print >> sys.stderr, "Cannot determine user name of "\
			"logged Mac OS X System user."
		return ""
	else:
		return macOSUserName[0].rstrip("\n")

# Since the script is executed as root, set ownership of fullPath 
# to user logged on console.
def setAccessRights( fullPath ):
	conUserName = getConsoleUser()
	print >> sys.stderr, "Setting " + fullPath + " owner to " + \
		conUserName + "."
	execute = "sudo chown " + conUserName + " " + \
		fullPath
	performOSCommand(execute)

# Return UUID.
def getGuid():
	guid = unicode( uuid.uuid4() ).upper()
	return guid

# Return domain/user/ path.
def getDomainUserPath():
	return KMSDomain + "/" + KMSUser + "/"

# Create new CalDAV directory for iCal.
def createNewiCalAccount():
	newCalDAVAccountFullName = "User " + getKMSUserNameAtDomain()
	newCalDAVAccountLogin = getKMSUserNameAtDomain()
	newCalDAVAccountPrincipalURL = getCalDAVPrincipalURL()
	newCalDAVAccountDisplay = getKMSUserNameAtDomain()
	
	sqlQueryInsertingNewCalDAVAccount = \
		"INSERT INTO Record VALUES(\'" + getGuid() +\
		"\',\'com.apple.calendars.CalDAVAccount\',55,0,\'com.apple.iCal\',55,8,0,0," +\
		"X'00000000AC010000A801000014000000090000001027000028000000000000000" +\
		"1000000370000000000000028000000010000003200000069040000000000000010" +\
		"2700002800000002000000010000003700000000000000280000000100000032000" +\
		"0006900000000000000001027000028000000030000000100000037000000000000" +\
		"0028000000010000003200000069000000000000000010270000280000000400000" +\
		"0010000003700000000000000280000000100000028000000050000000000000010" +\
		"2700002800000006000000010000003700000000000000280000000100000028000" +\
		"0000700000000000000102700002800000008000000010000003700000000000000" +\
		"280000000100000028000000090000000000000010270000280000000A000000010" +\
		"0000037000000000000002800000001000000320000007184030000000000000000" +\
		"000010270000280000000B000000010000003700000000000000280000000100000" +\
		"0DC0000000C0000000000000010270000280000000D000000010000003700000000" +\
		"0000002800000001000000280000000E00000000000000000000000F00000005000" +\
		"0006F726465720E000000636F6D2E6170706C652E6943616C0B0000007573654B65" +\
		"726265726F730F00000069734175746F446973636F766572790800000066756C6C4" +\
		"E616D65" +\
		intToStrHex( len( newCalDAVAccountFullName ) ) + "000000" + \
		strToStrHex( newCalDAVAccountFullName ) + \
		"27000000636F6D2E6170706C652E73796E6373657276696365732E5265636F72644" +\
		"56E746974794E616D6521000000636F6D2E6170706C652E63616C656E646172732E" +\
		"43616C4441564163636F756E74050000006C6F67696E" + \
		intToStrHex( len( newCalDAVAccountLogin ) ) + "000000" + \
		strToStrHex( newCalDAVAccountLogin ) + \
		"0F00000072656672657368496E74657276616C0C0000007072696E636970616C55524C" + \
		intToStrHex( len( newCalDAVAccountPrincipalURL ) ) + "000000" + \
		strToStrHex( newCalDAVAccountPrincipalURL ) + \
		"0B000000646973706C61794E616D65" + \
		intToStrHex( len( newCalDAVAccountDisplay ) ) + "000000" + \
		strToStrHex( newCalDAVAccountDisplay ) + \
		"\');"

	sqlFilePath = getHomeDir() + "Library/Application Support/SyncServices/Local/data.syncdb"
	sqlFilePath = sqlFilePath.replace(" ", "\ ")

	print >> sys.stderr, sqlQueryInsertingNewCalDAVAccount
	
	performOSCommand("echo \"" + sqlQueryInsertingNewCalDAVAccount + "\" | sqlite3 " + sqlFilePath)

# Configure SSL certificate in keychain.
def configureSecureHTTP():
	print >> sys.stderr, "Copying certificate to system keychain."
	KMScert = KMSHostName + ".cer"

	sourcePath = sys.path[0] + "/../../../../" + KMScert
	sourcePath = sourcePath.replace(" ", "\ ")

	performOSCommand("sudo security add-trusted-cert -d -p ssl -k\
		 \"/Library/Keychains/System.keychain\" " + sourcePath)
		 
def configureICalPasswordInKeychain(password):
	httpProtocolType = "http"
	if KMSHTTPisSSL == True:
		httpProtocolType = "htps"

	cmdPassword = password
	cmdPassword = cmdPassword.replace('"', r'\"')
	cmdPassword = cmdPassword.replace('$', r'\$')

	result = os.popen("sudo security add-internet-password -a \"" + getKMSUserNameAtDomain() + 
		"\" -s \"" + KMSHostName + "\" -p \"" + "/caldav/users/" + KMSDomain + "/" + KMSUser + "/" + 
		"\" -r "+ httpProtocolType + " -w " + cmdPassword + " -T \"/Applications/iCal.app/\" -c aapl -P " +
		 repr(KMSHTTPPort) + " -U 2>&1").readlines()

	if len( result ) > 0:
		print >> sys.stderr, "Access to KeyChain has been denied. In order to configure iCal CalDAV\
		 account password, access to KeyChain has to be allowed."
		return -1
	else:
		return 0

# Returns two-byte uppercase hex string representation of integer number.
def intToStrHex(i):
	strHex = str(hex(i)).upper()
	start = strHex.find("X") + 1
	stop = len(strHex)
	if stop == start+2:
		return strHex[start:stop]
	else:
		return "0" + strHex[start:stop]

# Each character in input string is replaced by its two-byte uppercase hex string 
# alternative.
def strToStrHex(str):
	strHex = ""
	for charAtIthPosition in str:
		hexChar = hex( ord( charAtIthPosition ) ).upper()
		start = hexChar.find("X") + 1
		stop = len(hexChar)
		if stop == start+2:
			strHex = strHex + hexChar[start:stop]
		else:
			strHex = strHex + "0" + hexChar[start:stop]

	return strHex

# Remove Calendar Cache file. It will force SyncServices to rebuild 
# the list of accounts.
def removeiCalCache():

	fullPath = getHomeDir() + CalendarDirectory
	cache = fullPath + "/Calendar Cache"
	try:
		os.remove(cache)
		os.remove(getHomeDir() + "Library/Caches/com.apple.iCal/Previous Recipients.plist")
		print >> sys.stderr, "iCal cache cleaned."
	except OSError:
		print >> sys.stderr, "Skipping iCal cache cleaning."

# Determine whether Mac OS sync is disabled.
def isMacOSSyncDisabled():

	MacOSSyncDisabled = False

	osCallResult = os.popen("ioreg -rd1 -c IOPlatformExpertDevice | perl " +\
		"-ne 'if (m/^.*\"IOPlatformUUID\" = \".*-(.*).*\"$/im) {print $1}'").readlines()

	computerUUID = ""
	if len( osCallResult ) > 0:
		computerUUID = osCallResult[0]

	iSyncConfigFilename = getHomeDir() + "Library/Preferences/ByHost/com.apple.syncservices." +\
		computerUUID.lower() + ".plist"
		
	if not os.path.exists(iSyncConfigFilename):
		fullPath = getHomeDir() + "Library/Preferences/ByHost"

		filesDirectoriesInCalendarsDirectory = os.listdir(fullPath)
		for theFileOrDirectory in filesDirectoriesInCalendarsDirectory:

			if "com.apple.syncservices." in theFileOrDirectory:
				iSyncConfigFilename = fullPath + "/" + theFileOrDirectory
				break

	iSyncConfigFilenameSlashed = iSyncConfigFilename.replace(" ", "\ ")

	# Python's plistlib can read only xml plists; convert binary-->xml
	installeISyncPlist = sys.path[0] + "/iSync.plist"
	installeISyncPlistSlashed = installeISyncPlist.replace(" ", "\ ")
	performOSCommand("rm " + installeISyncPlistSlashed)
	performOSCommand("cp " + iSyncConfigFilenameSlashed + " " + installeISyncPlistSlashed)

	performOSCommand("plutil -convert xml1 " + installeISyncPlistSlashed)

	try:
		print >> sys.stderr, "iSync config file:", iSyncConfigFilename
		iSyncConfigPlist = plistlib.Plist.fromFile(installeISyncPlist)
		syncingDisabledString = iSyncConfigPlist.get("SyncingDisabled")
		if syncingDisabledString.upper() == "YES":
			MacOSSyncDisabled = True
	except:
		print >> sys.stderr, "iSync config file not found. OK."

	debugString = "Mac OS X syncing is"
	if (MacOSSyncDisabled == True):
		print >> sys.stderr, debugString, "disabled."
	else:
		print >> sys.stderr, debugString, "enabled."

	return MacOSSyncDisabled

# Remove previous CalDAV directories created by this tool.
def removeCalDAVAccountTemplateDirectories():
	try:
		fullPath = getHomeDir() + CalendarDirectory

		filesDirectoriesInCalendarsDirectory = os.listdir(fullPath)
		for theFileOrDirectory in filesDirectoriesInCalendarsDirectory:

			if theFileOrDirectory.endswith(".caldav"):

				theFileOrDirectory = fullPath + "/" + theFileOrDirectory
				filesDirectoriesInCaldavDirectory = os.listdir(theFileOrDirectory)
				if (len( filesDirectoriesInCaldavDirectory ) < 3):
					performOSCommand("sudo rm -rf " + theFileOrDirectory)
		return

	except OSError:
		print >> sys.stderr, "Cannot list contents of Calendars directory."
		return

# Remove previous CalDAV directories created by this tool and directories
# based on them.
def removeCalDAVAccountCalendars():
	fullPath = getHomeDir() + CalendarDirectory

	if not os.path.exists(fullPath):
		os.mkdir(fullPath)
		setAccessRights(fullPath)

	try:
		filesDirectoriesInCalendarsDirectory = os.listdir(fullPath)
		for theFileOrDirectory in filesDirectoriesInCalendarsDirectory:

			if theFileOrDirectory.endswith(".caldav"):

				theFileOrDirectory = fullPath + "/" + theFileOrDirectory
				filesDirectoriesInCaldavDirectory = os.listdir(theFileOrDirectory)
				if (len( filesDirectoriesInCaldavDirectory ) < 3) or isThisAKMSAccount(\
					theFileOrDirectory):
					performOSCommand("sudo rm -rf " + theFileOrDirectory)
				else:

					for theCaldavFileOrDirectory in filesDirectoriesInCaldavDirectory:
						if theCaldavFileOrDirectory.endswith("Info.plist") == False:

							dir = theFileOrDirectory + "/" + theCaldavFileOrDirectory
							print >> sys.stderr, "Removing " + dir + " directory."
							performOSCommand("sudo rm -rf " + dir)
		return

	except OSError:
		print >> sys.stderr, "Cannot list contents of Calendars directory."
		return

# Create new iCal CalDAV account directory.		
def createDirectory():
	fullPath = getHomeDir() + CalendarDirectory

	fullPath = fullPath + "/" + getGuid() + ".caldav"
	os.mkdir( fullPath )

	setAccessRights( fullPath )

	return fullPath

# Create plist for iCal CalDAV account.
def createPList( accountDirectory ):	
	plistLocation = accountDirectory + "/Info.plist"
	
	iCalAccount = dict(
		AutoDiscovery=False,
		CalendarHomePath="/full-calendars/" + getDomainUserPath(),
		CalendarProxySupport=True,
		CalendarServerPrivateEventsSupport=True,
		CalendarUserAddresses=("mailto:" + getKMSUserNameAtDomain(),\
			 "/caldav/users/" + getDomainUserPath()),
		Color="#808080FF",
		Delegate=False,
		Expanded=True,
		FullName="User " + getKMSUserNameAtDomain(),
		InboxAvailabilitySupport=True,
		InboxPath="/full-calendars/" + getDomainUserPath() + "INBOX/",
		KeepOffline=False,
		Kerberos=False,
		Key=getGuid(),
		Login=getKMSUserNameAtDomain(),
		Order=3,
		OutboxPath="/full-calendars/" + getDomainUserPath() +\
			 "Sent%20Items/",
		PrincipalURL=getCalDAVPrincipalURL(),
		RefreshInterval=900,
		Title=getKMSUserNameAtDomain(),
		Type="CalDAV",
		VirtualHostKey=getGuid(),
		WebServicesRecordGUID=getGuid()
	)
	
	print >> sys.stderr, "Writing iCal configuration to " + plistLocation  + "."
	plistlib.writePlist(iCalAccount, plistLocation)

	setAccessRights( plistLocation )

# Contains principalURL KMSUserName and KMSHostName?
def getIsCalDAVPrincipalURL(principalURL):
	return principalURL.lower() == getCalDAVPrincipalURL().lower()

# Contains /Library/Calendars/theDirectory plist KMSUserName and KMSHostName?
def isThisAKMSAccount(theDirectory):
	fullpath = theDirectory + "/Info.plist"
	try:
		iCalPlist = plistlib.Plist.fromFile( fullpath )

		# Get PrincipalURL
		iCalPrincipalURL = iCalPlist.get( "PrincipalURL" )
		
		if ( getIsCalDAVPrincipalURL(iCalPrincipalURL) ):
			return True;
		else:
			return False;
	
	except IOError:
		print >> sys.stderr, "Error opening file " + fullpath + "."
		return False

# Create new CalDAV directory.
def configureCalDAVAccountWhenSyncingIsDisabled():
	
	print >> sys.stderr, "Creating new iCal CalDAV account in iCal directory."

	removeCalDAVAccountCalendars()
	
	newAccountDirectory = createDirectory()

	createPList( newAccountDirectory )

	removeiCalCache()

	return

# Create new SyncServices db entry.
def configureCalDAVAccountWhenSyncingIsEnabled():

	print >> sys.stderr, "Creating new iCal CalDAV account in SyncServices database."

	removeCalDAVAccountTemplateDirectories()

	createNewiCalAccount()

	removeiCalCache()

	print >> sys.stderr, "iCal Sync is being forced."
	performOSCommand("sudo -u " + getConsoleUser() +\
		" /System/Library/Frameworks/CalendarStore.framework/Resources/" +\
		"iCalExternalSync --sync com.apple.iCal --entitynames " +\
 		"com.apple.calendars.Attendee,com.apple.calendars.AudioAlarm," +\
		"com.apple.calendars.CalDAVAccount,com.apple.calendars.Calendar," +\
		"com.apple.calendars.CalendarOrder,com.apple.calendars.DisplayAlarm," +\
		"com.apple.calendars.Event,com.apple.calendars.MailAlarm,com.apple.calendars.Organizer," +\
		"com.apple.calendars.Recurrence,com.apple.calendars.Task," +\
		"com.apple.ical.calendars.DotMacPublisher,com.apple.ical.calendars.Node," +\
		"com.apple.ical.calendars.ProcAlarm,com.apple.ical.calendars.RootNode," +\
		"com.apple.ical.calendars.TaskOrder,com.apple.ical.calendars.URLPublisher")
	
	return

# CalDAV account configuration using SyncServices
from Foundation import *
from SyncServices import *
from objc import YES,  NO
from pwd import getpwnam
import sys
import random
import plistlib
import tempfile

caldavEntity = NSMutableString.stringWithString_(u"com.apple.calendars.CalDAVAccount")
client_name=NSMutableString.stringWithString_(u"com.kerio.Tool.Config.iCal")
tempFileForPlist = None
clientData = None
plistDict = {'Entities': {'com.apple.calendars.CalDAVAccount': ['com.apple.syncservices.RecordEntityName', 'order', 'useKerberos', 'principalURL', 'fullName', 'login', 'refreshInterval', 'isAutoDiscovery', 'displayName']}, 'Type': 'app', 'DisplayName': 'Kerio iCal Config Tool'}

def createPlistFile():
	global tempFileForPlist
	tempFileForPlist = tempfile.NamedTemporaryFile(suffix=".plist")
	plistlib.writePlist(plistDict,tempFileForPlist.name)
	return tempFileForPlist.name
	
def processChanges(session,changes):
	for change in changes:
		if change.type() == ISyncChangeTypeAdd:
			addRecord(session,change)

def addRecord(session,change):
	global clientData
	recId = change.recordIdentifier()
	record = change.record()
	localRecord = {} # need to copy into local dict so it will be mutable for updates
	localRecord.update(record);
	clientData[recId] = localRecord 
	print >> sys.stderr, "Adding record: %s" % repr(record)
	session.clientAcceptedChangesForRecordWithIdentifier_formattedRecord_newRecordIdentifier_(recId,None,None)

def performSync(client,changesToPush=None):
	global clientData
	print >> sys.stderr, "Synchronization started."
	pullTruth = NO
	if None == clientData:
		clientData = {}
		pullTruth = YES
		print >> sys.stderr, "Pulling the truth from SyncEngine."
	client.setShouldReplaceClientRecords_forEntityNames_(pullTruth, NSArray.arrayWithArray_([caldavEntity]))
	session = ISyncSession.beginSessionWithClient_entityNames_beforeDate_(client,NSArray.arrayWithArray_([caldavEntity]),NSDate.distantFuture())
	if changesToPush:
		print >> sys.stderr, "Pushing the truth to SyncEngine."
		for change in changesToPush:
			session.pushChange_(change)
	else:
		print >> sys.stderr, "No changes to SyncEngine."
	
	print >> sys.stderr, "Preparing to pull changes from SyncEngine."
	shouldPull = session.prepareToPullChangesForEntityNames_beforeDate_([caldavEntity],NSDate.distantFuture())
	if shouldPull == NO:
		print >> sys.stderr, "No changes for any entity when pulling from SyncEngine."
		session.finishSyncing()
		return
	shouldPull = session.shouldPullChangesForEntityName_(caldavEntity)
	if shouldPull == NO:
		print >> sys.stderr, "Should not pull. There are no changes for entity %" % (caldavEntity)
		session.finishSyncing()
		return
	changes_enum = session.changeEnumeratorForEntityNames_([caldavEntity])
	changes = changes_enum.allObjects()
	processChanges(session,changes)
	session.clientCommittedAcceptedChanges()
	session.finishSyncing()
	print >> sys.stderr, "Synchronization finished."

def createCaldavAccountRequest(caldav,id):
	changes = []
	for (key,val) in caldav.iteritems():
		mod = {}
		mod[ISyncChangePropertyActionKey] = ISyncChangePropertySet
		mod[ISyncChangePropertyNameKey] = key
		mod[ISyncChangePropertyValueKey] = val
		changes.append(mod)
	change = ISyncChange.changeWithType_recordIdentifier_changes_(ISyncChangeTypeAdd,id,changes)
	return change

def createCaldavId():
	return "PyObjcKerioRecord%d" % (random.randint(10000,20000))

def createCaldav():
	caldav = {}
	caldav["com.apple.syncservices.RecordEntityName"] = caldavEntity
	caldav["order"] = 3
	caldav["useKerberos"] = 0
	caldav["principalURL"] = NSURL.URLWithString_(getCalDAVPrincipalURL())
	caldav["fullName"] = "User " + getKMSUserNameAtDomain()
	caldav["login"]= getKMSUserNameAtDomain()
	caldav["refreshInterval"] = 900
	caldav["isAutoDiscovery"] = 0
	caldav["displayName"] = getKMSUserNameAtDomain();
	return caldav

def configureCalDAVAccountWithSyncServices():

	if os.getuid() == 0:
		# SyncService client does not work under root
		print >> sys.stderr, "Going to fork."
		pid = os.fork()
		if not pid:
			print >> sys.stderr, "Forked."
			userPw = pwd.getpwnam(getConsoleUser())
			userId = userPw.pw_uid
			os.setuid(userId)
			os.execlp("python", "python", "configure.py")
		
		print >> sys.stderr, "Master process waits."
		os.wait()[0]
		print >> sys.stderr, "Master process continues."
		return 

	else:
		# Connect to SyncService manager
		print >> sys.stderr, "Going to configure CalDAV account through SyncServices."
		manager = ISyncManager.sharedManager()
		if manager.isEnabled() != YES:
			print >> sys.stderr, "SyncEngine manager is not enabled. Finishing."
			return

		# Create a description of SyncService client
		plistFileName = createPlistFile()
		plistFileNameNS = NSMutableString.stringWithString_(plistFileName)
		client = manager.registerClientWithIdentifier_descriptionFilePath_(client_name,plistFileNameNS)

		# Pull the truth from SyncService
		performSync(client)

		# Create a new CalDAVAccount entry
		caldav = createCaldav()
		caldavId = createCaldavId()
		clientData[caldavId] = caldav
		changeRecord = createCaldavAccountRequest(caldav,caldavId)

		# Update the truth database with a new CalDAVAccount
		performSync(client,[changeRecord])
		print >> sys.stderr, "CalDAV account added through SyncServices."
	
		return

def main():
	print >> sys.stderr, "Child process started."
	configureCalDAVAccountWithSyncServices()
	print >> sys.stderr, "Child process finished."

if __name__ == '__main__':
	main()

# Main iCal configuration.
def configureiCal(loginName,password):

	res = 0
	if KMSHTTPisSSL == True:
		print >> sys.stderr, "Configuring keychain."
		configureSecureHTTP()
		print >> sys.stderr, "Keychain configured."
		
	if not isLeopard():
		print >> sys.stderr, "Configuring password in keychain."
		res = configureICalPasswordInKeychain(password)
		print >> sys.stderr, "Password in keychain configured."
	
	if isMacOSSyncDisabled() == True:
		configureCalDAVAccountWhenSyncingIsDisabled()
		
	else:
		if isLeopard():
			configureCalDAVAccountWhenSyncingIsEnabled()
		else:
			configureCalDAVAccountWithSyncServices()

	return res

def formatExceptionInfo(maxTBlevel=5):
	cla, exc, trbk = sys.exc_info()
	excName = cla.__name__
	try:
		excArgs = exc.__dict__["args"]
	except KeyError:
		excArgs = "<no args>"
		
	excTb = traceback.format_tb(trbk, maxTBlevel)
	return (excName, excArgs, excTb)
	
############################## Main script ################################

def configure(loginName, password):

	res = -1

	logFile = getHomeDir() + gLogFile
	sys.stderr = open(logFile, "a")

	try:
		print >> sys.stderr, "Kerio MailServer iCal/DS client configuration" +\
			" started."

		if os.getuid() != 0:
			print >> sys.stderr, "\nInsufficient privileges. You must run this" +\
				 " setup with administrator privileges."
		else:
			initializeUserAndDomain(loginName)
			res = tryConnect(password)
			if res == 0:
				print >> sys.stderr, "\nSource path: " + sys.path[0]

				if not (KMSLDAPPort == 0):
					print >> sys.stderr, "\nLDAP configuration:"
					res = configureDirectoryServices(loginName, password)
					print >> sys.stderr, "LDAP configuration finished."
					
				if not (KMSHTTPPort == 0):
					if (res == 0) or (res == -3):

						print >> sys.stderr, "\niCal configuration:"
						configureiCalResult = configureiCal(loginName, password)
						if configureiCalResult == -1:
							res = -4
												
						print >> sys.stderr, "iCal configuration finished."

		print >> sys.stderr, "\nKerio MailServer iCal/DS client configuration" +\
			" finished."
		
	except:
		print >> sys.stderr, "\nUnhandled exception caught.\n"
		print >> sys.stderr, formatExceptionInfo()
		res = -5

	sys.stderr.close()

	return res
