##########################################################################
#                                                                        #
#           (C) Kerio Technologies Inc. All rights reserved.             #
#                                                                        #
##########################################################################

import sys
from Foundation import *
from AppKit import NSApplicationMain
from PyObjCTools import NibClassBuilder
from AppKit import NSRunAlertPanel
from objc import *
from configure import *

NibClassBuilder.extractClasses('MainMenu.nib')

class Configurator (NibClassBuilder.AutoBaseClass):
	def autoconfig(self, loginName, password):		
		return configure(loginName, password)		

class ConfiguratorController (NibClassBuilder.AutoBaseClass):

	errCode = -1
	unableToReachMessage = "Unable to configure iCal and Directory Services"
	directoryServicesNotConfigured = "Cannot configure Directory Services"
	keychainDenied = "Access to Keychain forbidden"

	def awakeFromNib(self):
		self.serverHostNameLabel.setStringValue_(KMSHostName + ":" + repr(KMSHTTPPort))
		self.loginNameEdit.setStringValue_(KMSDisplayUser)
		self.passwordEdit.setStringValue_("")

		NSApplication.sharedApplication().windows()[0].center()
		NSApplication.sharedApplication().activateIgnoringOtherApps_(True)

		image = NSImage.imageNamed_('dialog')
		self.imageView.setImage_(image)

	def perform_(self, sender):
		loginName = self.loginNameEdit.stringValue()
		password = self.passwordEdit.stringValue()
		retval = self.configurator.autoconfig(loginName, password)
		if retval == 0:
			self.errCode = 0
			if (KMSLDAPPort == 0):
				NSRunAlertPanel(self.directoryServicesNotConfigured, "iCal CalDAV account "+ \
					"has been created. However, there are no LDAP ports available on "+\
					"Kerio MailServer. iCal autocompletion will not work.", "OK", None, None)
			self.closeWindow()
		elif retval == -1:
			self.errCode = -1
			NSRunAlertPanel(self.unableToReachMessage, "Incorrect user name/password.", \
				"OK", None, None)
		elif retval == -2:
			self.errCode = -1
			NSRunAlertPanel(self.unableToReachMessage, "Cannot connect to server.", \
				"OK", None, None)
		elif retval == -3:
			NSRunAlertPanel(self.directoryServicesNotConfigured, "Another" + \
				" Open Directory server is already configured with the Kerio MailServer hostname" + \
				" in Directory Utility. You will be able to use CalDAV account in iCal." + \
				" However, iCal autocompletion will not work.", "OK", None, None)
			self.errCode = 0
			self.closeWindow()

		elif retval == -4:
			NSRunAlertPanel(self.keychainDenied, "iCal" + \
				" CalDAV account and autocompletion has been configured." + \
				" Since you have denied access to Keychain, you can be required to enter" + \
				" mailbox password manually in iCal preferences.", "OK", None, None)
			self.errCode = 0
			self.closeWindow()

		elif retval == -5:
			self.errCode = -1
			NSRunAlertPanel("Unrecoverable error", "Unspecified" + \
				" problem emerged. Inspect" + \
				" ~/Library/Logs/kerio-ical-config-tool.log file for more information.", "OK", None, None)
						
	def cancel_(self, sender):
		self.closeWindow()
	
	def closeWindow(self):
		if not (self.errCode == 0):
			print "-1"
		NSApplication.sharedApplication().terminate_(self)

sys.exit(NSApplicationMain(sys.argv))
